Introduction      {#introduction}
=====

The ISEAFramework is a library for generic battery simulation and is developed at the Institute for Power Electronics and Electrical Drives (ISEA).
Models are based on datasets gained through electrochemical impedance spectroscopy (EIS) and a variety of different storage technologies can be described with these measurements,
e.g. lithium-ion batteries and double layer capacitors.
Results gained by EIS can be used to approximate the system through an electrical equivalent circuit and can be connected in different ways (parallel and serial).
This allows to describe storage systems in a generic way and to not only describe a single battery, but whole packs.
<br/>
Additionally a thermal model can be simulated, which simulates temperature distributions and trends inside the battery and battery pack.
This model is coupled with the electrical model during runtime, so that the thermal heat, which is generated by the electrical model, is directly fed into the thermal simulation.
<br/>
The framework also includes an aging model that can be used to make aging and lifetime predictions. The results from this model influence the electrical equivalent circuit so that the aging has an effect on the electrical behaviour of the battery.

Simulation sequence
====
A detailed description of the simulation process including the thermal and aging simulation can be found [here](simulation_cycle.html)


Initialization
----
- Creation of the electrical model
- Creation of the thermal model

Simulation cycle
-----
- 2. Set requested current
- 3. Update of lookup tables
- 4. Calculation of differential and algebraic equations (DAE)
- 5. Solving DAE system and calculating state space vector
- 6. Calculating currents, voltages and thermal losses
- 7. Passing the thermal losses (head) to the thermal model (not depicted in the image)
- 8. Calculating new temperature distribution and closing the feedback loop through passing the temperature back to the electrical model (not depicted in the image)

Cleanup
----
- Release allocated memory

At the end of the simulation the results can be plotted.


![Visualized results of a battery pack simulation](visualisierung2.png)


